// ------------------------------------------------------------------
// File.......: exp_add.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 10:44:27 CEST 2008
// Revision...: 1.0
// Description: Exponent adder. Adds the two inputs, and subtracts 
//              the bias.
// ------------------------------------------------------------------

`include "defines.v"

module exp_add 
(
	a,				// Input operand.
	b,				// Input operand.
	sum,			// Output sum.
	ovf_ab,		// Overflow after addition.
	ovf_biased	// Overflow after subtraction.
);
   
	parameter WIDTH = 1;
	parameter BIAS = `FP32BIAS;

 	// input(s)
	input [WIDTH-1:0]		a;
	input [WIDTH-1:0]		b;

	// output(s)
	output [WIDTH-1:0]	sum;
	output					ovf_ab;
	output					ovf_biased;

	// wire(s)	
	wire	[WIDTH:0]		a_plus_b_tmp;
	wire	[WIDTH:0]		biased_tmp;
		
	assign a_plus_b_tmp = a + b;
	assign biased_tmp = a_plus_b_tmp - BIAS;
	
	assign sum = biased_tmp[WIDTH-1:0];
	assign ovf_ab = a_plus_b_tmp[WIDTH];
	assign ovf_biased = biased_tmp[WIDTH];
		
endmodule // exp_add
