// ------------------------------------------------------------------
// File.......: mult_unit.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 11:37:56 CEST 2008
// Revision...: 1.0
// Description: Significand multiplier unit. 
// ------------------------------------------------------------------

`include "defines.v"

module mult_unit 
(
	fracs,	// Input from significand bus.
	format, 	// Input from instruction register.
	prods		// Output to significand bus.
);
   
 	// input(s)
	input [`FRACBUS-1:0]	fracs;
	input [1:0]				format;
	
	// output(s)
	output [`FRACBUS-1:0]	prods;

	// wire(s)
	wire [`FP16SW:0]		fp16_a_0;
	wire [`FP16SW:0]		fp16_b_0;
	wire [2*`FP16SW+1:0]	fp16_p_0;
	wire [`FP16SW:0]		fp16_a_1;
	wire [`FP16SW:0]		fp16_b_1;
	wire [2*`FP16SW+1:0]	fp16_p_1;
	wire [`FP32SW:0]		fp32_a_0;
	wire [`FP32SW:0]		fp32_b_0;
	wire [2*`FP32SW+1:0]	fp32_p_0;
	wire [`FP32SW:0]		fp32_a_1;
	wire [`FP32SW:0]		fp32_b_1;
	wire [2*`FP32SW+1:0]	fp32_p_1;
	wire [`FP64SW:0]		fp64_a_0;
	wire [`FP64SW:0]		fp64_b_0;
	wire [2*`FP64SW+1:0]	fp64_p_0;

	// reg(s)

	// -----------------------------------------------------------------------
	// Module instantiations.
	// -----------------------------------------------------------------------
	
	uns_mult #(`FP16SW+1) uns_mult_fp16_0
	(
		.a(fp16_a_0),
		.b(fp16_b_0),
		.p(fp16_p_0)
	);
	
	uns_mult #(`FP16SW+1) uns_mult_fp16_1
	(
		.a(fp16_a_1),
		.b(fp16_b_1),
		.p(fp16_p_1)
	);
	
	uns_mult #(`FP32SW+1) uns_mult_fp32_0
	(
		.a(fp32_a_0),
		.b(fp32_b_0),
		.p(fp32_p_0)
	);
	
	uns_mult #(`FP32SW+1) uns_mult_fp32_1
	(
		.a(fp32_a_1),
		.b(fp32_b_1),
		.p(fp32_p_1)
	);
	
	uns_mult #(`FP64SW+1) uns_mult_fp64_0
	(
		.a(fp64_a_0),
		.b(fp64_b_0),
		.p(fp64_p_0)
	);
	
	// ---------------------------------------------------------------
	// Combinaional assigns.
	// ---------------------------------------------------------------
	
	// Input demux.
	assign fp16_a_0 = 
		(format == `FP16) ? fracs[1*(`FP16SW+1)-1:0*(`FP16SW+1)] : 0;
	assign fp16_b_0 = 
		(format == `FP16) ? fracs[2*(`FP16SW+1)-1:1*(`FP16SW+1)] : 0;
	assign fp16_a_1 = 
		(format == `FP16) ? fracs[3*(`FP16SW+1)-1:2*(`FP16SW+1)] : 0;
	assign fp16_b_1 = 
		(format == `FP16) ? fracs[4*(`FP16SW+1)-1:3*(`FP16SW+1)] : 0;
	
	assign fp32_a_0 = 
		(format == `FP32) ? fracs[1*(`FP32SW+1)-1:0*(`FP32SW+1)] : 0;
	assign fp32_b_0 = 
		(format == `FP32) ? fracs[2*(`FP32SW+1)-1:1*(`FP32SW+1)] : 0;
	assign fp32_a_1 = 
		(format == `FP32) ? fracs[3*(`FP32SW+1)-1:2*(`FP32SW+1)] : 0;
	assign fp32_b_1 = 
		(format == `FP32) ? fracs[4*(`FP32SW+1)-1:3*(`FP32SW+1)] : 0;
	
	assign fp64_a_0 = 
		(format == `FP64) ? fracs[1*(`FP64SW+1)-1:0*(`FP64SW+1)] : 0;
	assign fp64_b_0 = 
		(format == `FP64) ? fracs[2*(`FP64SW+1)-1:1*(`FP64SW+1)] : 0;

	// Output mux.
	assign prods = 
		(format == `FP16) ? {fp16_p_1, fp16_p_0} :
		(format == `FP32) ? {fp32_p_1, fp32_p_0} :
		(format == `FP64) ? fp64_p_0 : 0;
	
endmodule // mult_unit
