// ------------------------------------------------------------------
// File.......: reg_excep.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 10:31:28 CEST 2008
// Revision...: 1.0
// Description: Exception register with synchronous reset, clear and 
//              enable.
// ------------------------------------------------------------------

`include "defines.v"

module reg_excep
(
	d,			// Data in.
	q,			// Data out.
	enable,	// Enable bit.
	clear,
	clk,		
	reset
);
   
	parameter WIDTH = `BUS;

 	// input(s)
	input [WIDTH-1:0]		d;
	input 					enable;
	input	[WIDTH-1:0]		clear;
	input 					clk;
	input						reset;

	// output(s)
	output [WIDTH-1:0]	q;

	// wire(s)
	
	// reg(s)
	reg [WIDTH-1:0]		q;
	
	always @ (posedge clk) begin
		if (reset) begin
			q <= 0;
		end
		else if (clear) begin
			q <= ~clear;
		end
		else if (enable) begin
			q <= d;
		end
	end
			
endmodule // reg_excep
