// ------------------------------------------------------------------
// File.......: stage1.v
// Author.....: Espen Stenersen
// Date.......: Fri Apr 18 16:11:23 CEST 2008
// Revision...: 1.0
// Description: Stage one in pipeline.
// ------------------------------------------------------------------

`include "defines.v"

module stage1 
(
 	vectors, 	// Input. Vectors.
	start,		// Input. Start computing.
	format,		// Input. Data format.
	mode,			// Input. Rounding mode.
	DRH0_out,	// Output. [127:64] of input.
	DRL0_out,	// Output. [63:0] of input.
	IR0_out,		// Output. Format.
	M0_out,		// Output. Rounding mode.
	clk,
	reset
);
   
 	// input(s)
	input [`BUS-1:0] 		vectors;
	input [0:0]				start;
	input [1:0]				format;
	input [1:0]				mode;
	input						clk;
	input						reset;

	// output(s)
	output [`BUS/2-1:0] 	DRH0_out;
	output [`BUS/2-1:0] 	DRL0_out;
	output [1:0] 			IR0_out;
	output [1:0] 			M0_out;

	// wire(s)
	wire						load_drh;
	wire						load_drl;
	wire						load_ir0;
	wire						load_m0;

	// reg(s)
	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------
	
	// Registers.
	reg_enable #(`BUS/2) DRH0
	(
		.d			(vectors[`BUS-1:`BUS/2]),	// Data in.
		.q			(DRH0_out),						// Data out.
		.enable	(load_drh),						// Enable bit.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	reg_enable #(`BUS/2) DRL0
	(
		.d			(vectors[`BUS/2-1:0]),	// Data in.
		.q			(DRL0_out),					// Data out.
		.enable	(load_drl),					// Enable bit.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	reg_enable #(2) M0
	(
		.d			(mode),		// Data in.
		.q			(M0_out),	// Data out.
		.enable	(load_m0),	// Enable bit.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	reg_enable #(2) IR0
	(
		.d			(format),		// Data in.
		.q			(IR0_out),		// Data out.
		.enable	(load_ir0),		// Enable bit.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	// ---------------------------------------------------------------
	// Assigns.
	// ---------------------------------------------------------------
	
	assign load_drh = start & (|format);
	assign load_drl = start;
	assign load_m0 = start;
	assign load_ir0 = start;

endmodule // stage1
