// ------------------------------------------------------------------
// File.......: stage2.v
// Author.....: Espen Stenersen
// Date.......: Fri Apr 18 16:29:31 CEST 2008
// Revision...: 1.0
// Description: Stage two of pipeline.
// ------------------------------------------------------------------

`include "defines.v"

module stage2 
(
	DRH0,			// Input from input register DRH0.
	DRL0,			// Input from input register DRL0.
	format,		// Input from format register IR0.
	mode,   		// Input from rounding mode register M0.
	DRF0_out,	// Output to significand multipliers.
	DRE0_out,	// Output to exponent adders.
	DRS0_out,	// Output to sign computation.
	IR1_out,		// Output.
	M1_out,		// Output.
	clk,
	reset
);
   
 	// input(s)
	input [`BUS/2-1:0]		DRH0;
	input [`BUS/2-1:0]		DRL0;
	input [1:0]					format;
	input [1:0]					mode;
	input							clk;
	input							reset;
	
	// output(s)
	output [`FRACBUS-1:0]	DRF0_out;
	output [`EXPBUS-1:0]		DRE0_out;
	output [`SIGNBUS-1:0]	DRS0_out;
	output [1:0]				IR1_out;
	output [1:0]				M1_out;

	// wire(s)
	wire [`FRACBUS-1:0]		fracs;
	wire [`EXPBUS-1:0]		exps;
	wire [`SIGNBUS-1:0]		signs;
	

	// reg(s)
	
	// ---------------------------------------------------------------
	// Module instantiations.
	// ---------------------------------------------------------------
	
	// Registers.
	ff #(`FRACBUS) DRF0
	(
		.d			(fracs),		// Data in.
		.q			(DRF0_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(`EXPBUS) DRE0
	(
		.d			(exps),		// Data in.
		.q			(DRE0_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(`SIGNBUS) DRS0
	(
		.d			(signs),		// Data in.
		.q			(DRS0_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);

	ff #(2) IR1
	(
		.d			(format),	// Data in.
		.q			(IR1_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	ff #(2) M1
	(
		.d			(mode),		// Data in.
		.q			(M1_out),	// Data out.
		.clk 		(clk),		
		.reset	(reset)
	);
	
	// Input mux / selector.
	sel_input sel_input
	(
		.drh		(DRH0),		// Input from data-register high (DRH0).
		.drl		(DRL0),		// Input from data-register low (DRL0).
		.format	(format),	// Input form instrucion(format) register.
		.signs	(signs),		// Output to sign bus.
		.exps		(exps),		// Output to exponent bus.
		.fracs	(fracs)		// Output to significand bus.
	);

	defparam sel_input.WIDTH 	= `BUS/2;
	defparam sel_input.SIGNBUS = `SIGNBUS;
	defparam sel_input.EXPBUS 	= `EXPBUS;
	defparam sel_input.FRACBUS = `FRACBUS;
	
endmodule // stage2
