// ------------------------------------------------------------------
// File.......: exp_add8.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 10:44:27 CEST 2008
// Revision...: 1.0
// Description: Exponent adder. Adds the two inputs, and subtracts 
//              the bias.
// ------------------------------------------------------------------

`include "defines.v"

module exp_add8 
(
	a,				// Input operand.
	b,				// Input operand.
	format,		// Input.
	sum,			// Output sum.
	ovf_ab,		// Overflow after addition.
	ovf_biased	// Overflow after subtraction.
);
   
	parameter WIDTH = `FP32EW;

 	// input(s)
	input [WIDTH-1:0]		a;
	input [WIDTH-1:0]		b;
	input [1:0]				format;

	// output(s)
	output [WIDTH-1:0]	sum;
	output					ovf_ab;
	output					ovf_biased;

	// wire(s)	
	wire	[WIDTH:0]		a_plus_b_tmp;
	wire	[WIDTH:0]		biased_tmp;
	
	// Exponent1 + exponent2	
	assign a_plus_b_tmp = a + b;
	
	// Subtract bias.
	assign biased_tmp = 
		(format == `FP16) ? a_plus_b_tmp - `FP16BIAS :
		(format == `FP32) ? a_plus_b_tmp - `FP32BIAS : 0;	
	// Selcet part of sum.
	assign sum = 
		(format == `FP16) ? biased_tmp[`FP16EW-1:0] : 
		(format == `FP32) ? biased_tmp[`FP32EW-1:0] : 0 ;
	
	// Compute overflow / underflow detection bits.	
	assign ovf_ab = 
		(format == `FP16) ? a_plus_b_tmp[`FP16EW] :
	   (format == `FP32) ? a_plus_b_tmp[`FP32EW] : 0;

	// Compute overflow / underflow detection bits.					
	assign ovf_biased = 
		(format == `FP16) ? biased_tmp[`FP16EW] :
	   (format == `FP32) ? biased_tmp[`FP32EW] :	 0;
	
		
endmodule // exp_add8
