// ------------------------------------------------------------------
// File.......: exp_unit.v
// Author.....: Espen Stenersen
// Date.......: Tue Apr 15 11:40:17 CEST 2008
// Revision...: 1.0
// Description: Exponent adder unit.
// ------------------------------------------------------------------

`include "defines.v"

module exp_unit 
(	
	exps,		// Input from exponent bus.
	format,	// Input from instruction register.
	sums		// Output to exponent bus.
);
   

 	// input(s)
	input [`EXPBUS-1:0]		exps;
	input [1:0]					format;

	// output(s)
	output [`EXPBUSOUT-1:0]	sums;

	// wire(s)
	wire [`FP32EW-1:0]		fp32_a_0;
	wire [`FP32EW-1:0]		fp32_b_0;
	wire [`FP32EW-1:0]		fp32_sum_0;
	wire 							fp32_ovf_ab_0;
	wire 							fp32_ovf_biased_0;
	wire [`FP64EW-1:0]		fp64_a_0;
	wire [`FP64EW-1:0]		fp64_b_0;
	wire [`FP64EW-1:0]		fp64_sum_0;
	wire 							fp64_ovf_ab_0;
	wire 							fp64_ovf_biased_0;
	
	// reg(s)
	
	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------
	
	exp_add8 #(`FP32EW) fp32_add_0
	(
		.a				(fp32_a_0),
		.b				(fp32_b_0),
		.sum			(fp32_sum_0),
		.format		(format),
		.ovf_ab		(fp32_ovf_ab_0),
		.ovf_biased	(fp32_ovf_biased_0)
	);
	
	exp_add11 #(`FP64EW) fp64_add_0
	(
		.a				(fp64_a_0),
		.b				(fp64_b_0),
		.format		(format),
		.sum			(fp64_sum_0),
		.ovf_ab		(fp64_ovf_ab_0),
		.ovf_biased	(fp64_ovf_biased_0)
	);
	
	// ---------------------------------------------------------------
	// Combinational assign.
	// ---------------------------------------------------------------
		
	// Input demux.

	assign fp32_a_0 = 
		(format == `FP16) ? 
		{{(`FP32EW-`FP16EW){1'b0}}, exps[1*`FP16EW-1:0*`FP16EW]} :
		(format == `FP32) ? 
		exps[1*`FP32EW-1:0*`FP32EW] : 0;
	assign fp32_b_0 = 
		(format == `FP16) ? 
		{{(`FP32EW-`FP16EW){1'b0}}, exps[2*`FP16EW-1:1*`FP16EW]} :
		(format == `FP32) ? 
		exps[2*`FP32EW-1:1*`FP32EW] : 0;
		
	assign fp64_a_0 = 
		(format == `FP16) ? 
		{{(`FP64EW-`FP16EW){1'b0}}, exps[3*`FP16EW-1:2*`FP16EW]} :
		(format == `FP32) ? 
		{{(`FP64EW-`FP32EW){1'b0}}, exps[3*`FP32EW-1:2*`FP32EW]} :
		(format == `FP64) ? 
		exps[1*`FP64EW-1:0*`FP64EW] : 0;
		
	assign fp64_b_0 = 
		(format == `FP16) ? 
		{{(`FP64EW-`FP16EW){1'b0}}, exps[4*`FP16EW-1:3*`FP16EW]} :
		(format == `FP32) ? 
		{{(`FP64EW-`FP32EW){1'b0}}, exps[4*`FP32EW-1:3*`FP32EW]} :
		(format == `FP64) ? 
		exps[2*`FP64EW-1:1*`FP64EW] : 0;
	
	// Output mux.
	assign sums = 
		(format == `FP16) ?
		{fp64_ovf_ab_0, fp64_ovf_biased_0, fp64_sum_0[`FP16EW-1:0],
		fp32_ovf_ab_0, fp32_ovf_biased_0, fp32_sum_0[`FP16EW-1:0]} : 
		(format == `FP32) ?
		{fp64_ovf_ab_0, fp64_ovf_biased_0, fp64_sum_0[`FP32EW-1:0],
		fp32_ovf_ab_0, fp32_ovf_biased_0, fp32_sum_0[`FP32EW-1:0]} : 
		(format == `FP64) ?
		{fp64_ovf_ab_0, fp64_ovf_biased_0, fp64_sum_0[`FP64EW-1:0]} : 
		0;
	
endmodule // exp_unit
