// ------------------------------------------------------------------
// File.......: sel_output.v
// Author.....: Espen Stenersen
// Date.......: Thu Apr 24 23:42:46 CEST 2008
// Revision...: 1.0
// Description: Loads the correct locations in output register and 
// 				 exception register.
// ------------------------------------------------------------------

`include "defines.v"

module sel_output 
(
	result,			// Input from rounding logic.
	exceps,			// Input from rounding logic.
	format,			// Input from format register.
	start,			// Input from start register.
	products,		// Output to output register.
	load_drh,		// Output to output register.
	load_drlh,		// Output to output register.
	load_drll,		// Output to output register.
	exceptions,		// Output to exception register.
	load_excep_l,	// Output to exception register.
	load_excep_h,	// Output to exception register.
	reset,
	clk
);
   

 	// input(s)
	input [`BUS/2-1:0]	result;
	input [7:0]				exceps;
	input [1:0]				format;
	input						start;
	input						clk;
	input						reset;

	// output(s)
	output [`BUS-1:0]		products;
	output [15:0]			exceptions;
	output 					load_drh;
	output 					load_drlh;
	output 					load_drll;
	output 					load_excep_l;
	output 					load_excep_h;


	// wire(s)

	// reg(s)
	reg [`BUS-1:0]			products;
	reg [15:0]				exceptions;
	reg 						load_drh;
	reg 						load_drlh;
	reg 						load_drll;
	reg 						load_excep_l;
	reg 						load_excep_h;
	reg						counter;
	

	always @ (posedge clk) begin
		if (reset) begin
			counter <= 0;
		end
		else begin
			if (start) begin
				counter <= counter + 1;
			end
			else begin
				counter <= 0;
			end
		end
	end
		
	always @ (result or exceps or format or counter or start) begin
		products = 0;
		exceptions = 0;
		load_drh = 0;
		load_drlh = 0;
		load_drll = 0;
		load_excep_l = 0;
		load_excep_h = 0;
		
		case (format)
			`FP16: begin
				case (counter)
					0: begin
						products[31:0] = result[31:0];
						exceptions[7:0] = exceps;
						load_drll = 1;
						load_excep_l = 1;
					end
					1: begin
						products[63:32] = result[31:0];
						exceptions[15:8] = exceps;
						load_drlh = 1;
						load_excep_h = 1;
					end
				endcase
			end
			`FP32: begin
				case (counter)
					0: begin
						products[63:0] = result[63:0];
						exceptions[7:0] = exceps;
						load_drll = 1;
						load_drlh = 1;
						load_excep_l = 1;
					end
					1: begin
						products[127:64] = result[63:0];
						exceptions[15:8] = exceps;
						load_drh = 1;
						load_excep_h = 1;
					end
				endcase
			end
			`FP64: begin
				case (counter)
					0: begin
						products[63:0] = result[63:0];
						exceptions[7:0] = exceps;
						load_drll = 1;
						load_drlh = 1;
						load_excep_l = 1;
					end
					1: begin
						products[127:64] = result[63:0];
						exceptions[15:8] = exceps;
						load_drh = 1;
						load_excep_h = 1;
					end
				endcase
			end
		endcase
	end
		
endmodule // sel_output