// ------------------------------------------------------------------
// File.......: dw_vec_fp16_mult.v
// Author.....: Espen Stenersen
// Date.......: Thu Apr 24 16:40:38 CEST 2008
// Revision...: 1.0
// Description: Vectorized FP16 floating-point multiplier based on 
//              the DesignWare simulation model.
// ------------------------------------------------------------------

`include "defines.v"

module dw_vec_fp_mult 
(
	dw_vectors,		// Input from testbench.
	dw_mode,			// Input from testbench.
	format,		// Input from testbench.
	dw_products,	// Output to testbench.
	dw_exceptions	// Output to testbench.
);
  

 	// input(s)
	input [2*`BUS-1:0]				dw_vectors;
	input [2:0]							dw_mode;
	input [1:0]							format;

	// output(s)
	output [`BUS-1:0]					dw_products;
	output [15:0]						dw_exceptions;

	// wire(s)
	wire [7:0]							fp16_mult0_status;
	wire [7:0]							fp16_mult1_status;
	wire [7:0]							fp16_mult2_status;
	wire [7:0]							fp16_mult3_status;
	wire [`FP16SW+`FP16EW:0]		fp16_mult0_z;
	wire [`FP16SW+`FP16EW:0]		fp16_mult1_z;
	wire [`FP16SW+`FP16EW:0]		fp16_mult2_z;
	wire [`FP16SW+`FP16EW:0]		fp16_mult3_z;
	wire [`FP16SW+`FP16EW:0]		fp16_mult0_a;
	wire [`FP16SW+`FP16EW:0]		fp16_mult0_b;
	wire [`FP16SW+`FP16EW:0]		fp16_mult1_a;
	wire [`FP16SW+`FP16EW:0]		fp16_mult1_b;
	wire [`FP16SW+`FP16EW:0]		fp16_mult2_a;
	wire [`FP16SW+`FP16EW:0]		fp16_mult2_b;
	wire [`FP16SW+`FP16EW:0]		fp16_mult3_a;
	wire [`FP16SW+`FP16EW:0]		fp16_mult3_b;
	wire [`FP16SW+`FP16EW:0]		fp16_mult0_z_tmp;
	wire [`FP16SW+`FP16EW:0]		fp16_mult1_z_tmp;
	wire [`FP16SW+`FP16EW:0]		fp16_mult2_z_tmp;
	wire [`FP16SW+`FP16EW:0]		fp16_mult3_z_tmp;
	
	wire [7:0]							fp32_mult0_status;
	wire [7:0]							fp32_mult1_status;
	wire [7:0]							fp32_mult2_status;
	wire [7:0]							fp32_mult3_status;
	wire [`FP32SW+`FP32EW:0]		fp32_mult0_z;
	wire [`FP32SW+`FP32EW:0]		fp32_mult1_z;
	wire [`FP32SW+`FP32EW:0]		fp32_mult2_z;
	wire [`FP32SW+`FP32EW:0]		fp32_mult3_z;
	wire [`FP32SW+`FP32EW:0]		fp32_mult0_a;
	wire [`FP32SW+`FP32EW:0]		fp32_mult0_b;
	wire [`FP32SW+`FP32EW:0]		fp32_mult1_a;
	wire [`FP32SW+`FP32EW:0]		fp32_mult1_b;
	wire [`FP32SW+`FP32EW:0]		fp32_mult2_a;
	wire [`FP32SW+`FP32EW:0]		fp32_mult2_b;
	wire [`FP32SW+`FP32EW:0]		fp32_mult3_a;
	wire [`FP32SW+`FP32EW:0]		fp32_mult3_b;
	wire [`FP32SW+`FP32EW:0]		fp32_mult0_z_tmp;
	wire [`FP32SW+`FP32EW:0]		fp32_mult1_z_tmp;
	wire [`FP32SW+`FP32EW:0]		fp32_mult2_z_tmp;
	wire [`FP32SW+`FP32EW:0]		fp32_mult3_z_tmp;

	wire [7:0]							fp64_mult0_status;
	wire [7:0]							fp64_mult1_status;
	wire [`FP64SW+`FP64EW:0]		fp64_mult0_z;
	wire [`FP64SW+`FP64EW:0]		fp64_mult1_z;
	wire [`FP64SW+`FP64EW:0]		fp64_mult0_a;
	wire [`FP64SW+`FP64EW:0]		fp64_mult0_b;
	wire [`FP64SW+`FP64EW:0]		fp64_mult1_a;
	wire [`FP64SW+`FP64EW:0]		fp64_mult1_b;
	wire [`FP64SW+`FP64EW:0]		fp64_mult0_z_tmp;
	wire [`FP64SW+`FP64EW:0]		fp64_mult1_z_tmp;
	
	// reg(s)
	

	// ---------------------------------------------------------------
	// Module instantiation.
	// ---------------------------------------------------------------
	
	dw_fp_mult #(`FP16SW, `FP16EW, 1) fp16_mult0
	(
		.a			(fp16_mult0_a), 
		.b			(fp16_mult0_b), 
		.rnd		(dw_mode), 
		.z			(fp16_mult0_z), 
		.status	(fp16_mult0_status)
	);
	dw_fp_mult #(`FP16SW, `FP16EW, 1) fp16_mult1
	(
		.a			(fp16_mult1_a), 
		.b			(fp16_mult1_b), 
		.rnd		(dw_mode), 
		.z			(fp16_mult1_z), 
		.status	(fp16_mult1_status)
	);
	dw_fp_mult #(`FP16SW, `FP16EW, 1) fp16_mult2
	(
		.a			(fp16_mult2_a), 
		.b			(fp16_mult2_b), 
		.rnd		(dw_mode), 
		.z			(fp16_mult2_z), 
		.status	(fp16_mult2_status)
	);
	dw_fp_mult #(`FP16SW, `FP16EW, 1) fp16_mult3
	(
		.a			(fp16_mult3_a), 
		.b			(fp16_mult3_b), 
		.rnd		(dw_mode), 
		.z			(fp16_mult3_z), 
		.status	(fp16_mult3_status)
	);
	
	dw_fp_mult #(`FP32SW, `FP32EW, 1) fp32_mult0
	(
		.a			(fp32_mult0_a), 
		.b			(fp32_mult0_b), 
		.rnd		(dw_mode), 
		.z			(fp32_mult0_z), 
		.status	(fp32_mult0_status)
	);
	dw_fp_mult #(`FP32SW, `FP32EW, 1) fp32_mult1
	(
		.a			(fp32_mult1_a), 
		.b			(fp32_mult1_b), 
		.rnd		(dw_mode), 
		.z			(fp32_mult1_z), 
		.status	(fp32_mult1_status)
	);
	dw_fp_mult #(`FP32SW, `FP32EW, 1) fp32_mult2
	(
		.a			(fp32_mult2_a), 
		.b			(fp32_mult2_b), 
		.rnd		(dw_mode), 
		.z			(fp32_mult2_z), 
		.status	(fp32_mult2_status)
	);
	dw_fp_mult #(`FP32SW, `FP32EW, 1) fp32_mult3
	(
		.a			(fp32_mult3_a), 
		.b			(fp32_mult3_b), 
		.rnd		(dw_mode), 
		.z			(fp32_mult3_z), 
		.status	(fp32_mult3_status)
	);
	
	dw_fp_mult #(`FP64SW, `FP64EW, 1) fp64_mult0
	(
		.a			(fp64_mult0_a), 
		.b			(fp64_mult0_b), 
		.rnd		(dw_mode), 
		.z			(fp64_mult0_z), 
		.status	(fp64_mult0_status)
	);
	dw_fp_mult #(`FP64SW, `FP64EW, 1) fp64_mult1
	(
		.a			(fp64_mult1_a), 
		.b			(fp64_mult1_b), 
		.rnd		(dw_mode), 
		.z			(fp64_mult1_z), 
		.status	(fp64_mult1_status)
	);
	
	
	// ---------------------------------------------------------------
	// Input selections.
	// ---------------------------------------------------------------
	assign fp16_mult0_a = dw_vectors[1*`FP16W-1:0*`FP16W];
	assign fp16_mult0_b = dw_vectors[2*`FP16W-1:1*`FP16W];
	assign fp16_mult1_a = dw_vectors[3*`FP16W-1:2*`FP16W];
	assign fp16_mult1_b = dw_vectors[4*`FP16W-1:3*`FP16W];
	assign fp16_mult2_a = dw_vectors[5*`FP16W-1:4*`FP16W];
	assign fp16_mult2_b = dw_vectors[6*`FP16W-1:5*`FP16W];
	assign fp16_mult3_a = dw_vectors[7*`FP16W-1:6*`FP16W];
	assign fp16_mult3_b = dw_vectors[8*`FP16W-1:7*`FP16W];
	assign fp32_mult0_a = dw_vectors[1*`FP32W-1:0*`FP32W];
	assign fp32_mult0_b = dw_vectors[2*`FP32W-1:1*`FP32W];
	assign fp32_mult1_a = dw_vectors[3*`FP32W-1:2*`FP32W];
	assign fp32_mult1_b = dw_vectors[4*`FP32W-1:3*`FP32W];
	assign fp32_mult2_a = dw_vectors[5*`FP32W-1:4*`FP32W];
	assign fp32_mult2_b = dw_vectors[6*`FP32W-1:5*`FP32W];
	assign fp32_mult3_a = dw_vectors[7*`FP32W-1:6*`FP32W];
	assign fp32_mult3_b = dw_vectors[8*`FP32W-1:7*`FP32W];
	assign fp64_mult0_a = dw_vectors[1*`FP64W-1:0*`FP64W];
	assign fp64_mult0_b = dw_vectors[2*`FP64W-1:1*`FP64W];
	assign fp64_mult1_a = dw_vectors[3*`FP64W-1:2*`FP64W];
	assign fp64_mult1_b = dw_vectors[4*`FP64W-1:3*`FP64W];
	
	
	
	// ---------------------------------------------------------------
	// Set exceptions.
	// ---------------------------------------------------------------
	
	// Invalid. dw_status[2].
	assign dw_exceptions[0] =  
		(format == `FP16) ? 
	   fp16_mult0_status[2] :
	   (format == `FP32) ? 
		fp32_mult0_status[2] :
		(format == `FP64) ? 
		fp64_mult0_status[2] : 
		0;
										
	assign dw_exceptions[1] = 	
		(format == `FP16) ? 
	   fp16_mult1_status[2] :
	   (format == `FP32) ? 
		fp32_mult1_status[2] :
		(format == `FP64) ? 
		fp64_mult1_status[2] : 
		0;
										
	assign dw_exceptions[2] = 	
		(format == `FP16) ? 
	   fp16_mult2_status[2] :
	   (format == `FP32) ? 
		fp32_mult2_status[2] :
		(format == `FP64) ? 
		0 : 0;
										
	assign dw_exceptions[3] = 	
		(format == `FP16) ? 
	   fp16_mult3_status[2] :
	   (format == `FP32) ? 
		fp32_mult3_status[2] :
		(format == `FP64) ? 
		0 : 0;

	// Inexact. dw_status[5].
	assign dw_exceptions[4] = 
		(format == `FP16) ? 
		fp16_mult0_status[5]|fp16_mult0_status[3] :
		(format == `FP32) ? 
		fp32_mult0_status[5]|fp32_mult0_status[3] :
	   (format == `FP64) ? 
		fp64_mult0_status[5]|fp64_mult0_status[3] : 
		0;
										
	assign dw_exceptions[5] = 	
		(format == `FP16) ? 
		fp16_mult1_status[5]|fp16_mult1_status[3] :
		(format == `FP32) ? 
		fp32_mult1_status[5]|fp32_mult1_status[3] :
		(format == `FP64) ? 
		fp64_mult1_status[5]|fp64_mult1_status[3] : 
		0;
										
	assign dw_exceptions[6] = 	
		(format == `FP16) ? 
		fp16_mult2_status[5]|fp16_mult2_status[3] :
		(format == `FP32) ? 
		fp32_mult2_status[5]|fp32_mult2_status[3] :
	   (format == `FP64) ? 
		0 : 0;
										
	assign dw_exceptions[7] = 	
		(format == `FP16) ? 
	   fp16_mult3_status[5]|fp16_mult3_status[3] :
	   (format == `FP32) ? 
		fp32_mult3_status[5]|fp32_mult3_status[3] :
		(format == `FP64) ? 
		0 : 0;
										
	// Overflow. dw_status[1].
	assign dw_exceptions[8] =  
		(format == `FP16) ? 
	   fp16_mult0_status[1] :
	   (format == `FP32) ? 
		fp32_mult0_status[1] :
		(format == `FP64) ? 
		fp64_mult0_status[1] : 
		0;
										
	assign dw_exceptions[9] = 	
		(format == `FP16) ? 
	   fp16_mult1_status[1] :
	   (format == `FP32) ? 
		fp32_mult1_status[1] :
		(format == `FP64) ? 
		fp64_mult1_status[1] : 
		0;
										
	assign dw_exceptions[10] = 
		(format == `FP16) ? 
	   fp16_mult2_status[1] :
	   (format == `FP32) ? 
		fp32_mult2_status[1] :
		(format == `FP64) ? 
		0 : 0;
										
	assign dw_exceptions[11] = 
		(format == `FP16) ? 
	   fp16_mult3_status[1] :
	   (format == `FP32) ? 
		fp32_mult3_status[1] :
		(format == `FP64) ? 
		0 : 0;
										
	// Underflow. dw_status[0] | dw_status[3] (underflow|denormal).
	assign dw_exceptions[12] = 
		(format == `FP16) ? 
	   fp16_mult0_status[0]|fp16_mult0_status[3] :
	   (format == `FP32) ? 
		fp32_mult0_status[0]|fp32_mult0_status[3] :
		(format == `FP64) ? 
		fp64_mult0_status[0]|fp64_mult0_status[3] : 
		0;
										
	assign dw_exceptions[13] = 
		(format == `FP16) ? 
	   fp16_mult1_status[0]|fp16_mult1_status[3] :
	   (format == `FP32) ? 
		fp32_mult1_status[0]|fp32_mult1_status[3] :
		(format == `FP64) ? 
		fp64_mult1_status[0]|fp64_mult1_status[3] : 
		0;
										
	assign dw_exceptions[14] = 
		(format == `FP16) ? 
	   fp16_mult2_status[0]|fp16_mult2_status[3] :
	   (format == `FP32) ? 
		fp32_mult2_status[0]|fp32_mult2_status[3] :
 		(format == `FP64) ? 
		0 : 0;

	assign dw_exceptions[15] = 
		(format == `FP16) ? 
	   fp16_mult3_status[0]|fp16_mult3_status[3] :
	   (format == `FP32) ? 
		fp32_mult3_status[0]|fp32_mult3_status[3] :
		(format == `FP64) ? 
		0 : 0;
	
	// Flush product to zero if denormal output from dw_dp_mult.
	assign fp16_mult0_z_tmp = fp16_mult0_status[3] ? 
		{fp16_mult0_z[`FP16W-1], fp16_mult0_z[`FP16W-2:0]&1'b0} : 
		fp16_mult0_z;

	assign fp16_mult1_z_tmp = fp16_mult1_status[3] ?
		{fp16_mult1_z[`FP16W-1], fp16_mult1_z[`FP16W-2:0]&1'b0} : 
		fp16_mult1_z;

	assign fp16_mult2_z_tmp = fp16_mult2_status[3] ?
		{fp16_mult2_z[`FP16W-1], fp16_mult2_z[`FP16W-2:0]&1'b0} : 
		fp16_mult2_z;

	assign fp16_mult3_z_tmp = fp16_mult3_status[3] ?
		{fp16_mult3_z[`FP16W-1], fp16_mult3_z[`FP16W-2:0]&1'b0} : 
		fp16_mult3_z;
								
	assign fp32_mult0_z_tmp = fp32_mult0_status[3] ? 
		{fp32_mult0_z[`FP32W-1], fp32_mult0_z[`FP32W-2:0]&1'b0} : 
		fp32_mult0_z;

	assign fp32_mult1_z_tmp = fp32_mult1_status[3] ?
		{fp32_mult1_z[`FP32W-1], fp32_mult1_z[`FP32W-2:0]&1'b0} : 
		fp32_mult1_z;

	assign fp32_mult2_z_tmp = fp32_mult2_status[3] ?
		{fp32_mult2_z[`FP32W-1], fp32_mult2_z[`FP32W-2:0]&1'b0} : 
		fp32_mult2_z;

	assign fp32_mult3_z_tmp = fp32_mult3_status[3] ?
		{fp32_mult3_z[`FP32W-1], fp32_mult3_z[`FP32W-2:0]&1'b0} : 
		fp32_mult3_z;
								
	assign fp64_mult0_z_tmp = fp64_mult0_status[3] ? 
		{fp64_mult0_z[`FP64W-1], fp64_mult0_z[`FP64W-2:0]&1'b0} : 
		fp64_mult0_z;

	assign fp64_mult1_z_tmp = fp64_mult1_status[3] ?
		{fp64_mult1_z[`FP64W-1], fp64_mult1_z[`FP64W-2:0]&1'b0} : 
		fp64_mult1_z;
								

	// ---------------------------------------------------------------
	// Output mux.
	// ---------------------------------------------------------------
	assign dw_products = (format == `FP16) ? 
		{fp16_mult3_z_tmp, fp16_mult2_z_tmp, 
		fp16_mult1_z_tmp, fp16_mult0_z_tmp} :
		(format == `FP32) ? 
		{fp32_mult3_z_tmp, fp32_mult2_z_tmp, 
		fp32_mult1_z_tmp, fp32_mult0_z_tmp} :
		(format == `FP64) ? 
		{fp64_mult1_z_tmp, fp64_mult0_z_tmp} : 0;
endmodule // dw_vec_fp_mult
